/* Rappel :
 *  un bit a 0 dans un masque = bit toujours accepte, pas de filtrage
 *  un bit a 1 dans masque
 *    . si bit filtre a 0 => bit a 0 du message accepte, bit a 1 refuse
 *    . si bit filtre a 1 => bit a 1 du message accepte, bit a 0 refuse
 * il y a deux masques et six flitres
 *    . le masque 0 (RXM0) est associ� aux flitres 0 et 1 (RXF0 et RXF1) pour le buffer de reception 0 (RXB0)
 *    . le masque 1 (RXM1) est associ� aux filtres 2 a 5 (RXF2 a RXF5)pour le buffer de reception 1 (RXB1)
 */

#ifndef COM_H
#define COM_H

/**************************************************************************
 * LES BIBLIOTHEQUES
 **************************************************************************/

#include <SPI.h>                    // la biblioth�que SPI
#include "mcp_can.h"              // la biblioth�que CAN

#define NB_MESSAGES 5            // taille de la file d'attente des messages arrivant du bus CAN
#define TENTATIVE_CONNECTION 10     // nombre de tentative de connection du bus
#define MASQUE 0xFFF              // on ne prend que l'ID du TCO

// DEFINITION D'UN MESSAGE CAN  //
struct messageCAN {
  long unsigned int id;                          // ident du message
  unsigned char len;                            // longueur des donnees
  unsigned char buf[MAX_CHAR_IN_MESSAGE];      // buffer de donnees
};

/*********************************************************
classe qui sert d'interface simplifiee pour les besoins du
passage a niveau.
Le passage ne recoit seulement les messages qui lui sont
adresses (mask et ID fixes)
constructeur :
    CS : la broche CS  (10 pour NANO/MINI/UNO, 53 pour MEGA)
    baudRate : le debit du bus (50, 100, 250, 500, 1000)
    idmode : 0 normal, 1 etendu
Methodes :
        bool init(int filtre, int ID): initialise le bus CAN
        void receptionMessage() : reception des messages depuis le bus
        bool lireCode(unsigned char& code) : reception d'un code depuis le TCO
        int ecrireCode(unsigned char code ) : envoie d'un code vers le TCO
*******************************************************************/

class COM : MCP_CAN
{
    public:
        // le constructeur
        COM(int cs, int baudRate, int idmode);

        // le destructeur
        virtual ~COM();

        // initialisation simplifiee du bus CAN
        bool init(int flitre, int ID);

        // reception des messages depuis le bus
        void receptionMessage();

        // reception d'un code depuis le TCO
        bool lireCode(unsigned char&);

        // envoie d'un code vers le TCO
        int ecrireCode(unsigned char);

    protected:
    private:
        // les parametres du bus CAN
        int CS;
        int debit;
        int ID_mode;
        int IDlocal;
        int IDdistant;

        // initialisation complete du bus CAN
        bool init(int mask0, int filter0, int filter1, int mask1, int filter2, int filter3, int filter4, int filter5);

        // LE BUFFER DE RECEPTION DES MESSAGES CAN //
        int indexEcriture = 0;                             // index d'ecriture
        int indexLecture = 0;                              // index de lecture
        messageCAN BufferReception[NB_MESSAGES];          // buffer circulaire de reception
};

#endif // CANBUS_H
