#include "bobine.h"

// le constructeur

bobine::bobine() : barriere() {}

/***********************************************************************************
parametres :
    ouvre : pin pour commande ouverture
    ferme : pin pour commande fermeture
    cde  : niveau actif de commande des bobines (HIGH ou LOW)
    activation : duree de la manoeuvre
    delai  : temps d'attente avant ouverture du passage apr�s passage d'un train
****************************************************************************/

bobine::bobine(int pina, int pinb,bool cde, int activation, int delai) : barriere(activation,2),pin1(pina), pin2(pinb)
{   _cde = cde;
    attente = delai;            // delai avant d'ouvrir la barriere
    sens = arret;               // initialement a l'arret
    situation = ouvert;         // initialement ouverte
    _actif = false;              // initialement pas d'activite
    pinMode(pin1,OUTPUT); digitalWrite(pin1,!_cde);
    pinMode(pin2,OUTPUT); digitalWrite(pin2,!_cde);
}

// le destructeur
bobine::~bobine(){}

/*****************************************************
OUVERTURE : methode d'initialisation de l'ouverture du passage a niveau
***************************************************************/

void bobine::ouverture() {
if (!ouverte()){                // si pas ouverte
    _actif = true;               // on est actif
    timer = millis();           // on demarre le temps de manoeuvre
    sens = montee;              // on monte la barriere
    situation = encours;        // manoeuvre en cours
    }
}

/***************************************************************************
FERMETURE : methode d'initialisation de fermeture du passage a niveau
NB : dans le cas des bobines la fermeture est instantan�e
**************************************************************************/

void bobine::fermeture() {
if (!fermee()){                 // si pas ferme
    _actif = true;               // on est actif
    sens = descente;            // on descent la barriere
    situation = encours;        // manoeuvre en cours
    }
}

/****************************************************************
MANOEUVRE : phases de manoeuvre de la barriere (ouverture ou fermeture)
************************************************************************/

void bobine::manoeuvre() {
if (_actif){                                 // si actif
    if (sens == descente) {                 // si descente
            digitalWrite(pin2,_cde);    // on alimente la bobine de fermeture
            delay(duree);                   // on attend un delai
            digitalWrite(pin2,!_cde);   // on arrete la bobine de fermeture
            _actif = false;                  // on n'est plus actif
            situation = ferme;              // la barriere est fermee
            sens = arret;                   // on est arrete
            }
    else {                                  // sinon on monte
        if ((timer + attente) < millis()) { // on attend la periode  d'attente apres le passage du train
                digitalWrite(pin1,_cde); // on alimente la bobine d'ouverture
                delay(duree);               // on attend un delai
                digitalWrite(pin1,!_cde);// on arrete la bobine d'ouverture
                _actif = false;              // on n'est plus actif
                situation = ouvert;         // on est ouvert
                sens = arret;               // on est arrete
                }
           }
    }
}
